#!/usr/bin/env python -3
# -*- coding: utf-8 -*-
#
# Copyright 2010 K. Richard Pixley.
# See LICENSE for details.
#
# Time-stamp: <03-Jan-2011 17:21:27 PST by rich@noir.com>

"""
A covering script for :py:mod:`elffile`.  Compare object files.
"""

from __future__ import unicode_literals, print_function

__docformat__ = 'restructuredtext en'

import glob
import itertools
import optparse
import pprint
import sys

import elffile

if __name__ == '__main__':

    progname = sys.argv[0]
    u = ''
    u += 'usage: %prog objfile1 objfile2'

    parser = optparse.OptionParser(usage = u)

    options, args = parser.parse_args()

    assert len(args) == 2

    x = elffile.open(name=args[0])
    y = elffile.open(name=args[1])
    
    if x.close_enough(y):
        sys.exit()

    print('different')

    sys.exit(1)



import yaml
import elffile
import sys
import glob
import os
import itertools

def filelist(which):
    for f in glob.iglob(os.path.join('testfiles', which, '.libs', 'hello')):
        yield f

    for f in glob.iglob(os.path.join('testfiles', which, '.libs', '*.so*')):
        yield f

    for f in glob.iglob(os.path.join('testfiles', which, '.libs', '*.o')):
        yield f

    for f in glob.iglob(os.path.join('testfiles', which, '*.o')):
        yield f

for xname, yname in itertools.izip(filelist('one'), filelist('two')):

    x = elffile.open(name=xname)
    y = elffile.open(name=yname)

    print('{0}: {1}'.format(xname, x.close_enough(y)))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2010 K. Richard Pixley.
# See LICENSE for details.
#
# Time-stamp: <02-Jan-2011 19:31:41 PST by rich@noir.com>

"""
Tests for elffile.

.. todo:: need to set up a distributable example files directory.
"""

from __future__ import unicode_literals, print_function

__docformat__ = 'restructuredtext en'

import nose
from nose.tools import assert_true, assert_false, assert_equal, assert_raises, raises

import glob
import sys
import os
import mmap

import elffile

def testOpen():
    for filename in glob.glob(os.path.join('testfiles', '*', '*.o')):
        break

    byname = elffile.open(name=filename)

    with open(filename, 'rb') as fileobj:
        byfileobj = elffile.open(name=filename, fileobj=fileobj)

    with open(filename, 'rb') as fileobj:
        m = mmap.mmap(fileobj.fileno(), 0, mmap.MAP_SHARED, mmap.PROT_READ)
        bymap = elffile.open(map=m)

        block = m[:]

    byblock = elffile.open(block=block)

    assert_equal(byname, byfileobj)
    assert_equal(byfileobj, bymap)
    assert_equal(bymap, byblock)


def testTestfiles():
    for filename in (glob.glob(os.path.join('testfiles', '*', '*.o'))
                     + glob.glob(os.path.join('testfiles', '*', '*', '*.o'))
                     + glob.glob(os.path.join('testfiles', '*', '*.so*'))
                     + glob.glob(os.path.join('testfiles', '*', '.libs', '*.so*'))
                     + glob.glob(os.path.join('testfiles', '*', 'hello'))):
        with open(filename, 'rb') as f:
            content = f.read()

        efi = elffile.ElfFileIdent()
        efi.unpack_from(content)

        if efi.magic != '\x7fELF':
            continue

        #print('{0}: {1}'.format(f.name, efi), file=sys.stderr)
        newcontent = bytearray(efi.size)
        efi.pack_into(newcontent)

        # if content != newcontent:
        #     print('len(content) = {0}, len(newcontent) = {1}'.format(len(content), len(newcontent)))

        #     for i in xrange(len(content)):
        #         if content[i] != newcontent[i]:
        #             print('differs at char {0}: {1} != {2}'.format(i, content[i], newcontent[i]))

        assert_true(content.startswith(newcontent))

        efi2 = elffile.ElfFileIdent()
        efi2.unpack_from(bytes(newcontent))
        assert_equal(efi, efi2)

        ef = elffile.ElfFile(filename, efi)
        ef.unpack_from(content)

        newcontent = bytearray(ef.size)
        ef.pack_into(newcontent)

        # FIXME: this is only true if we pack with the same order.  :\.  Maybe need 3 stage?
        #assert_true(content.startswith(newcontent)) # FIXME: eventually need to compare equality

        ef2 = elffile.ElfFile(filename, efi2)
        ef2.unpack_from(bytes(newcontent))
        assert_equal(ef, ef2)


def testFileEncoding():
    for i in elffile._fileEncodingDict:
        for j in elffile._fileEncodingDict[i]:
            ident = elffile.ElfFileIdent()
            ident.elfClass = i
            ident.elfData = j
            assert_equal(elffile._fileEncodingDict[i][j], elffile.ElfFile.encodedClass(ident))

@raises(elffile.ElfFile.NO_CLASS)
def testBogusClass():
    ident = elffile.ElfFileIdent()
    ident.elfClass = 254
    elffile.ElfFile.encodedClass(ident)

@raises(elffile.ElfFile.NO_ENCODING)
def testBogusEncoding():
    ident = elffile.ElfFileIdent()

    for ident.elfClass in elffile._fileEncodingDict:
        break

    ident.elfData = 254
    elffile.ElfFile.encodedClass(ident)


if __name__ == '__main__':
    nose.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2010 K. Richard Pixley.
# See LICENSE for details.
#
# Time-stamp: <13-Jan-2011 19:53:03 PST by rich@noir.com>

import os

# try:
#     from setuptools import setup, find_packages
# except ImportError:
#     from ez_setup import use_setuptools
#     use_setuptools()
#     from setuptools import setup, find_packages

import distribute_setup
distribute_setup.use_setuptools()

from setuptools import setup

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name='elffile',
    version='0.002',
    author='K. Richard Pixley',
    author_email='rich@noir.com',
    description='A pure python library for reading and writing ELF format object filex.',
    license='MIT',
    keywords='elf object file',
    url='http://bitbucket.org/krp/elffile',
    long_description=read(os.path.join('README.rst')),
    setup_requires=[
    	'nose>=1.0.0',
#        'sphinx>=1.0.5',
    ],
    install_requires=[
        'coding',
    ],
    py_modules=['elffile'],
    test_suite='nose.collector',
    scripts = [
    	'objdump.py',
        'objcmp.py',
    ],
    requires=[
    ],
    provides=[
        'elffile',
    ],
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.0',
        'Programming Language :: Python :: 3.1',
        'Programming Language :: Python :: 3.2',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
